@@ -41,25 +41,28 @@ static int atmodem_init(void)
 	at_call_meter_init();
 	at_call_settings_init();
 	at_phonebook_init();
-	at_ssn_init();
 	at_ussd_init();
 	at_sms_init();
 	at_sim_init();
+	at_stk_init();
 	at_netreg_init();
 	at_cbs_init();
 	at_call_volume_init();
 	at_gprs_init();
 	at_gprs_context_init();
+	at_sim_auth_init();
+	at_gnss_init();
 
 	return 0;
 }
 
 static void atmodem_exit(void)
 {
+	at_sim_auth_exit();
+	at_stk_exit();
 	at_sim_exit();
 	at_sms_exit();
 	at_ussd_exit();
-	at_ssn_exit();
 	at_phonebook_exit();
 	at_call_settings_exit();
 	at_call_meter_exit();
@@ -72,6 +75,7 @@ static void atmodem_exit(void)
 	at_call_volume_exit();
 	at_gprs_exit();
 	at_gprs_context_exit();
+	at_gnss_exit();
 }
 
 OFONO_PLUGIN_DEFINE(atmodem, "AT modem driver", VERSION,
